#include "include/tool_call_agent.h"
#include "mcp/include/mcp_tool.h"
#include <iostream>
#include <sstream>

namespace openmanus {

ToolCallAgent::ToolCallAgent(const std::string& name, const std::string& description, const std::string& config_file)
    : AgentBase(name, description), config_(config_file), max_steps_(30), current_step_(0), should_clear_history_(false), max_consecutive_tool_calls_(5), consecutive_tool_calls_(0) {
    // 从配置文件中读取LLM API相关信息
    llm_provider_ = config_.getString("llm", "provider", "local");
    
    if (llm_provider_ == "local") {
        // 本地服务器配置
        llm_api_host_ = config_.getString("llm", "host", "localhost");
        llm_api_port_ = config_.getInt("llm", "port", 8000);
        llm_api_endpoint_ = config_.getString("llm", "path", "/chat/completions");
        
        // 初始化HTTP客户端 - 本地服务器
        http_client_ = std::make_unique<httplib::Client>(llm_api_host_, llm_api_port_);
    } else {
        // 云服务提供商配置
        llm_api_base_url_ = config_.getString("llm", "base_url", "");
        llm_api_endpoint_ = config_.getString("llm", "path", "/v1/chat/completions");
        
        // 初始化HTTP客户端 - 云服务
        http_client_ = std::make_unique<httplib::Client>(llm_api_base_url_);
    }
    
    // 通用配置
    llm_api_key_ = config_.getString("llm", "api_key", "");
    llm_model_ = config_.getString("llm", "model", "gpt-3.5-turbo");
    llm_max_tokens_ = config_.getInt("llm", "max_tokens", 4096);
    
    // 设置HTTP客户端超时
    http_client_->set_connection_timeout(300); // 设置连接超时时间为5分钟
    http_client_->set_read_timeout(300);       // 设置读取超时时间为5分钟
    
    // 从配置文件中读取代理相关信息
    max_steps_ = config_.getInt("agent", "max_steps", 30);
    max_consecutive_tool_calls_ = config_.getInt("agent", "max_consecutive_tool_calls", 5);
    system_prompt_ = config_.getString("agent", "system_prompt", "你是一个能够使用工具的智能助手。");
    next_step_prompt_ = config_.getString("agent", "next_step_prompt", "请思考下一步行动。");
    should_clear_history_ = config_.getBool("agent", "clear_history_each_run", false);
    
    // 添加系统提示
    if (!system_prompt_.empty()) {
        messages_.push_back(Message::systemMessage(system_prompt_));
    }
}

void ToolCallAgent::setLLMAPIParams(const std::string& host, int port, const std::string& endpoint) {
    // 设置为本地服务器模式
    llm_provider_ = "local";
    llm_api_host_ = host;
    llm_api_port_ = port;
    llm_api_endpoint_ = endpoint;
    
    // 重新初始化HTTP客户端
    http_client_ = std::make_unique<httplib::Client>(llm_api_host_, llm_api_port_);
    http_client_->set_connection_timeout(300); // 设置连接超时时间为5分钟
    http_client_->set_read_timeout(300);       // 设置读取超时时间为5分钟
}

// 新增方法，用于设置云服务提供商的API参数
void ToolCallAgent::setCloudLLMAPIParams(const std::string& provider, const std::string& base_url, const std::string& endpoint) {
    llm_provider_ = provider;
    llm_api_base_url_ = base_url;
    llm_api_endpoint_ = endpoint;
    
    // 重新初始化HTTP客户端
    http_client_ = std::make_unique<httplib::Client>(llm_api_base_url_);
    http_client_->set_connection_timeout(300); // 设置连接超时时间为5分钟
    http_client_->set_read_timeout(300);       // 设置读取超时时间为5分钟
}

void ToolCallAgent::setLLMAPIKey(const std::string& api_key) {
    llm_api_key_ = api_key;
}

void ToolCallAgent::clearMessages() {
    messages_.clear();
    
    // 添加系统提示
    if (!system_prompt_.empty()) {
        messages_.push_back(Message::systemMessage(system_prompt_));
    }
}

void ToolCallAgent::addMessage(const Message& message) {
    messages_.push_back(message);
}

std::string ToolCallAgent::run(const std::string& prompt) {
    // 清空工具调用
    tool_calls_.clear();
    current_step_ = 0;
    consecutive_tool_calls_ = 0;  // 重置连续工具调用计数器
    
    // 如果配置为每次运行清空历史，则清空
    if (should_clear_history_) {
        clearMessages();
    }
    
    // 添加用户提示到消息历史
    addMessage(Message::userMessage(prompt));
    
    std::string result;
    bool terminated = false;
    
    // 执行代理循环
    while (current_step_ < max_steps_ && !terminated) {
        // 思考下一步行动
        bool continue_execution = think();
        if (!continue_execution) {
            break;
        }
        
        // 检查是否有Terminate工具调用
        for (const auto& tool_call : tool_calls_) {
            if (tool_call.getName() == "Terminate") {
                std::cout << "🛑 检测到Terminate工具调用，将在此步骤后停止执行" << std::endl;
                terminated = true;
                break;
            }
        }
        
        // 执行行动
        result = act();
        
        // 检查是否应该结束执行
        if (terminated || shouldFinishExecution()) {
            break;
        }
        
        current_step_++;
    }
    
    return result;
}

bool ToolCallAgent::think() {
    // 检查连续工具调用次数
    if (consecutive_tool_calls_ >= max_consecutive_tool_calls_) {
        std::cout << "⚠️ 达到最大连续工具调用次数限制(" << max_consecutive_tool_calls_ << ")，强制停止" << std::endl;
        
        // 添加一条消息，告知LLM已达到工具调用限制
        std::string limit_message = "已达到最大连续工具调用次数限制(" + std::to_string(max_consecutive_tool_calls_) + 
                                   ")。请总结目前的进展并提供最终答案，不要再使用工具。";
        addMessage(Message::userMessage(limit_message));
        
        // 重置计数器
        consecutive_tool_calls_ = 0;
    }
    
    // 如果有下一步提示，添加到消息历史
    if (!next_step_prompt_.empty()) {
        addMessage(Message::userMessage(next_step_prompt_));
    }
    
    try {
        // 调用LLM API获取下一步行动
        mcp::json response = callLLMAPI(messages_);
        
        // 解析响应，提取工具调用
        tool_calls_.clear();
        
        if (response.contains("choices") && response["choices"].is_array() && !response["choices"].empty()) {
            auto& choice = response["choices"][0];
            
            // 记录思考内容
            std::string content = "";
            if (choice.contains("message") && choice["message"].contains("content")) {
                content = choice["message"]["content"].get<std::string>();
                if (!content.empty()) {
                    std::cout << "✨ " << name_ << "的思考: " << content << std::endl;
                    
                    // 添加助手消息到历史
                    addMessage(Message::assistantMessage(content));
                }
            }
            
            // 提取工具调用
            if (choice.contains("message") && choice["message"].contains("tool_calls") && 
                choice["message"]["tool_calls"].is_array()) {
                
                auto& tool_calls_json = choice["message"]["tool_calls"];
                std::cout << "🛠️ " << name_ << "选择了 " << tool_calls_json.size() << " 个工具使用" << std::endl;
                
                if (!tool_calls_json.empty()) {
                    std::cout << "🧰 正在准备的工具: ";
                    bool has_terminate = false;
                    
                    for (const auto& tool_call_json : tool_calls_json) {
                        if (tool_call_json.contains("id") && tool_call_json.contains("function")) {
                            std::string id = tool_call_json["id"];
                            std::string name = tool_call_json["function"]["name"];
                            
                            // 检查是否是Terminate工具
                            if (name == "Terminate") {
                                has_terminate = true;
                                std::cout << "🛑 检测到Terminate工具调用!" << std::endl;
                            }
                            
                            mcp::json params = tool_call_json["function"].contains("arguments") ? 
                                               tool_call_json["function"]["arguments"] : mcp::json::object();
                            
                            // 如果参数是字符串，尝试解析为JSON
                            if (params.is_string()) {
                                try {
                                    params = mcp::json::parse(params.get<std::string>());
                                } catch (...) {
                                    // 如果解析失败，保持原样
                                }
                            }
                            
                            // 创建工具调用对象
                            ToolCall tool_call(id, name, params);
                            tool_calls_.push_back(tool_call);
                            
                            std::cout << name << " ";
                        }
                    }
                    std::cout << std::endl;
                }
            }
        }
        
        return !tool_calls_.empty();
    } catch (const std::exception& e) {
        std::cerr << "思考过程中出错: " << e.what() << std::endl;
        return false;
    }
}

mcp::json ToolCallAgent::callLLMAPI(const std::vector<Message>& messages) {
    // 构建请求JSON
    mcp::json request;
    
    // 根据不同的LLM提供商构建不同的请求
    if (llm_provider_ == "deepseek") {
        // DeepSeek API请求格式
        request["model"] = llm_model_;
        request["messages"] = mcp::json::array();
        request["max_tokens"] = llm_max_tokens_;
        
        // 添加消息
        for (const auto& message : messages) {
            mcp::json msg_json;
            
            // 设置角色
            switch (message.role) {
                case MessageRole::SYSTEM:
                    msg_json["role"] = "system";
                    break;
                case MessageRole::USER:
                    msg_json["role"] = "user";
                    break;
                case MessageRole::ASSISTANT:
                    msg_json["role"] = "assistant";
                    break;
                case MessageRole::TOOL:
                    // 工具消息现在被转换为用户消息，所以这里不应该有工具消息
                    std::cerr << "警告：发现工具消息，但我们现在将工具消息转换为用户消息" << std::endl;
                    continue;  // 跳过这个消息
            }
            
            msg_json["content"] = message.content;
            request["messages"].push_back(msg_json);
        }
        
        // 添加工具定义
        if (available_tools_ && available_tools_->size() > 0) {
            request["tools"] = mcp::json::array();
            
            for (const auto& tool_name : available_tools_->getToolNames()) {
                ToolBase* tool = available_tools_->getTool(tool_name);
                if (tool) {
                    // 构建工具JSON
                    mcp::json tool_json;
                    tool_json["type"] = "function";
                    tool_json["function"]["name"] = tool->getName();
                    tool_json["function"]["description"] = tool->getDescription();
                    tool_json["function"]["parameters"] = tool->getParameters();
                    
                    request["tools"].push_back(tool_json);
                }
            }
            
            // 设置工具选择策略
            request["tool_choice"] = "auto";
        }
    } else {
        // 默认OpenAI兼容格式
        request["model"] = llm_model_;
        request["messages"] = mcp::json::array();
        
        // 添加消息
        for (const auto& message : messages) {
            mcp::json msg_json;
            
            // 设置角色
            switch (message.role) {
                case MessageRole::SYSTEM:
                    msg_json["role"] = "system";
                    break;
                case MessageRole::USER:
                    msg_json["role"] = "user";
                    break;
                case MessageRole::ASSISTANT:
                    msg_json["role"] = "assistant";
                    break;
                case MessageRole::TOOL:
                    // 工具消息现在被转换为用户消息，所以这里不应该有工具消息
                    std::cerr << "警告：发现工具消息，但我们现在将工具消息转换为用户消息" << std::endl;
                    msg_json["role"] = "user";
                    break;
            }
            
            msg_json["content"] = message.content;
            request["messages"].push_back(msg_json);
        }
        
        // 添加工具定义
        if (available_tools_ && available_tools_->size() > 0) {
            request["tools"] = mcp::json::array();
            
            for (const auto& tool_name : available_tools_->getToolNames()) {
                ToolBase* tool = available_tools_->getTool(tool_name);
                if (tool) {
                    // 构建工具JSON
                    mcp::json tool_json;
                    tool_json["type"] = "function";
                    tool_json["function"]["name"] = tool->getName();
                    tool_json["function"]["description"] = tool->getDescription();
                    tool_json["function"]["parameters"] = tool->getParameters();
                    
                    request["tools"].push_back(tool_json);
                }
            }
            
            // 设置工具选择策略
            request["tool_choice"] = "auto";
        }
    }

    // freopen("request.json", "w", stdout);
    // std::cout << request.dump(2) << std::endl;
    // fclose(stdout);
    
    // 发送请求
    std::string request_str = request.dump();
    httplib::Headers headers = {
        {"Content-Type", "application/json"}
    };
    
    // 添加Authorization头部
    if (!llm_api_key_.empty()) {
        headers.emplace("Authorization", "Bearer " + llm_api_key_);
    }
    
    auto response = http_client_->Post(llm_api_endpoint_, headers, request_str, "application/json");
    
    if (response && response->status == 200) {
        // 解析响应
        mcp::json response_json = mcp::json::parse(response->body);
        return response_json;
    } else {
        std::string error_msg = "LLM API调用失败";
        if (response) {
            error_msg += ": " + std::to_string(response->status) + " " + response->body;
        }
        throw std::runtime_error(error_msg);
    }
}

std::string ToolCallAgent::act() {
    std::string result;
    
    // 如果有工具调用，增加计数器
    if (!tool_calls_.empty()) {
        consecutive_tool_calls_++;
        std::cout << "🔄 连续工具调用次数: " << consecutive_tool_calls_ << "/" << max_consecutive_tool_calls_ << std::endl;
    } else {
        // 如果没有工具调用，重置计数器
        consecutive_tool_calls_ = 0;
    }
    
    // 执行所有工具调用
    for (auto& tool_call : tool_calls_) {
        std::string tool_result = executeToolCall(tool_call);
        tool_call.setResult(tool_result);
        
        // 处理特殊工具
        if (isSpecialTool(tool_call.getName())) {
            handleSpecialTool(tool_call.getName(), tool_result);
        }
        
        // 将工具结果添加为用户消息，使用明确的格式
        std::string tool_message = "工具执行结果 [" + tool_call.getName() + "]: \n```json\n" + tool_result + "\n```\n请根据这个结果继续你的任务。";
        // std::cout << "📝 添加消息到历史: " << tool_message << std::endl;
        addMessage(Message::userMessage(tool_message));
        
        result += tool_result + "\n";
    }
    
    // 打印当前消息历史长度
    std::cout << "📚 当前消息历史长度: " << messages_.size() << std::endl;

    // 清空工具调用列表，准备下一轮
    tool_calls_.clear();
    
    return result;
}

std::string ToolCallAgent::executeToolCall(const ToolCall& tool_call) {
    std::string name = tool_call.getName();
    mcp::json params = tool_call.getParams();
    
    std::cout << "🔧 执行工具: " << name << " 参数: " << params.dump(2) << std::endl;
    
    // 获取工具
    ToolBase* tool = available_tools_->getTool(name);
    if (!tool) {
        std::string error_msg = "错误：找不到工具 " + name;
        std::cout << "❌ " << error_msg << std::endl;
        return error_msg;
    }
    
    try {
        // 执行工具
        mcp::json result = tool->execute(params);
        
        // 格式化结果，确保它是一个有效的JSON字符串
        std::string result_str;
        try {
            // 如果结果已经是一个JSON对象，则直接格式化
            result_str = result.dump(2);
        } catch (...) {
            // 如果结果不是一个有效的JSON对象，则将其包装为一个JSON对象
            mcp::json wrapper;
            wrapper["result"] = result.dump();
            result_str = wrapper.dump(2);
        }
        
        std::cout << "✅ 工具执行结果: " << result_str << std::endl;
        return result_str;
    } catch (const std::exception& e) {
        std::string error_msg = "错误：执行工具 " + name + " 失败：" + e.what();
        std::cout << "❌ " << error_msg << std::endl;
        return error_msg;
    }
}

bool ToolCallAgent::handleSpecialTool(const std::string& name, const std::string& result) {
    // 处理特殊工具，如终止工具
    if (name == "Terminate") {
        std::cout << "🛑 收到终止指令，停止执行" << std::endl;
        // 添加一条消息，表示任务已完成
        addMessage(Message::assistantMessage("任务已完成，执行已终止。"));
        return true;
    }
    return false;
}

bool ToolCallAgent::shouldFinishExecution() {
    // 检查是否应该结束执行
    
    // 检查是否达到最大步骤数
    if (current_step_ >= max_steps_) {
        std::cout << "⚠️ 达到最大步骤数限制(" << max_steps_ << ")，停止执行" << std::endl;
        return true;
    }
    
    // 如果连续工具调用次数超过限制，也应该结束执行
    if (consecutive_tool_calls_ >= max_consecutive_tool_calls_) {
        std::cout << "⚠️ 达到最大连续工具调用次数限制(" << max_consecutive_tool_calls_ << ")，停止执行" << std::endl;
        return true;
    }
    
    return false;
}

bool ToolCallAgent::isSpecialTool(const std::string& name) {
    // 检查是否是特殊工具
    // 目前只有Terminate是特殊工具
    if (name == "Terminate") {
        std::cout << "🔍 识别到特殊工具: Terminate" << std::endl;
        return true;
    }
    return false;
}

} // namespace openmanus 